﻿Imports System.ComponentModel
Imports System.Runtime.CompilerServices


Public MustInherit Class BaseViewModel
    Implements INotifyPropertyChanged

    Private Shared _DepObj As New DependencyObject
    Public Shared Function IsInDesignMode() As Boolean
        Return DesignerProperties.GetIsInDesignMode(_DepObj)
    End Function

    Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

    Protected Function ChangePropIfDifferent(Of T)(value As T, ByRef backingField As T, <CallerMemberName> Optional name As String = "") As Boolean
        If Object.Equals(backingField, value) Then Return False
        backingField = value
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(name))
        Return True
    End Function
    Protected Function ChangePropIfDifferent(Of T)(value As T, ByRef backingField As T, ParamArray propNames() As String) As Boolean
        Return ChangePropIfDifferent(value, backingField, propNames.Select(Function(s) New PropertyChangedEventArgs(s)).ToArray)
    End Function
    Protected Function ChangePropIfDifferent(Of T)(value As T, ByRef backingField As T, ParamArray props() As PropertyChangedEventArgs) As Boolean
        If Object.Equals(backingField, value) Then Return False
        backingField = value
        RaisePropChanged(props)
        Return True
    End Function
    Public Sub RaisePropChanged(Optional name As String = "")
        RaisePropChanged({New PropertyChangedEventArgs(name)})
    End Sub
    Public Sub RaisePropChanged(name As String, ParamArray names() As String)
        Dim names2(names.Length) As String
        names2(0) = name : Array.Copy(names, 0, names2, 1, names.Length)
        RaisePropChanged(names2.Select(Function(nm) New PropertyChangedEventArgs(nm)).ToArray)
    End Sub

    Public Sub RaisePropChanged(ParamArray eventArgses() As PropertyChangedEventArgs)
        RaisePropChanged(DirectCast(eventArgses, IEnumerable(Of PropertyChangedEventArgs)))
    End Sub
    Public Sub RaisePropChanged(eventArgses As IEnumerable(Of PropertyChangedEventArgs))
        VerifyPropertyNames(eventArgses)
        For Each e In eventArgses
            RaiseEvent PropertyChanged(Me, e)
        Next
    End Sub
    Public Sub RaiseAllPropChanged()
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(Nothing))
    End Sub

    Private Shared _AllProperties As New Dictionary(Of Type, HashSet(Of String))
    <Conditional("DEBUG"), DebuggerStepThrough()>
    Public Sub VerifyPropertyNames(eventArgses As IEnumerable(Of PropertyChangedEventArgs))
        Dim propNames As HashSet(Of String) = Nothing
        Dim tp = Me.GetType
        If Not _AllProperties.TryGetValue(tp, propNames) Then
            propNames = New HashSet(Of String)(From pd In Me.GetType.GetProperties Select pd.Name)
            _AllProperties.Add(tp, propNames)
        End If
        For Each pe In eventArgses
            If Not propNames.Contains(pe.PropertyName) Then Throw New Exception("trying to raise an unknown PropertyChanged")
        Next
    End Sub

#If False Then
   Private Shared ReadOnly HostProcesses As New List(Of String)({"XDesProc", "devenv", "WDExpress"})
   Private _isBusy As Boolean

    ''' <summary>
    ''' Gibt zurück ob sich die ausführung des Codes aktuell in der Entwicklungszeit oder in der Laufzeit befindet.
    ''' Für eine Logausgabe muss das LogAction Propertie genutzt werden.
    ''' </summary>
    ''' <returns>Wird Code des ViewModels vom XAML Designer ausgeführt wird True zurückgegeben.</returns>
    <Browsable(False)>
    Public ReadOnly Property IsInDesignMode As Boolean
        Get
            Dim ret = HostProcesses.Contains(Process.GetCurrentProcess().ProcessName)
            Return ret
        End Get
    End Property


   Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
   ''' <summary>
   ''' Prozedur wirft den INotifyPropertyChanged Event welcher in der WPF benötigt wird um die UI zu verständingen
   ''' das eine Änderung an einem Property stattgefunden hat.
   ''' </summary>
   ''' <param name="prop">Das Propertie welches sich geändert hat. Ist Optional da als Parameter "CallerMemberName" verwendet wird. Wird Nothing übergeben werden alle PRoperties des Views aktualisiert!!</param>
   Protected Overridable Sub RaisePropertyChanged(<CallerMemberName> Optional ByVal prop As String = "")
        If prop Is Nothing OrElse prop.Length > 0 Then RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(prop))
    End Sub

#End If

End Class