﻿Imports System.Collections.ObjectModel


Public Class MainViewModel
    Inherits BaseViewModel

    Public Shared ReadOnly Instance As New MainViewModel

    Private __AvailableCountries As ObservableCollection(Of CountryViewModel) = CountryViewModel.Seed
    Public WithEvents AvailableCountries As New ListCollectionView(__AvailableCountries) ' enables access/observe the currently selected Person
    Private Sub AvailableCountries_CurrentChanged(sender As Object, e As EventArgs) Handles AvailableCountries.CurrentChanged
        CalcResults()
    End Sub

    Public Sub New()
        AvailableCountries.MoveCurrentToPosition(3)
    End Sub

    'False = Service True = Products
    Private _salestype As Boolean = True
    Public Property SalesType() As Boolean
        Get
            Return _salestype
        End Get
        Set(ByVal value As Boolean)
            ChangePropIfDifferent(value, _salestype)
            CalcResults()
        End Set
    End Property


    Private _taxuse As Boolean = True
    Public Property TaxUse() As Boolean
        Get
            Return _taxuse
        End Get
        Set(ByVal value As Boolean)
            ChangePropIfDifferent(value, _taxuse)
            CalcResults()
        End Set
    End Property








    Private _daybookset As String = " - "
    Public Property DayBookSet() As String
        Get
            Return _daybookset
        End Get
        Set(ByVal value As String)
            ChangePropIfDifferent(value, _daybookset)
        End Set
    End Property


    Private _taxusage As Boolean = True
    Public Property TaxUsage As Boolean
        Get
            Return _taxusage
        End Get
        Set(ByVal value As Boolean)
            ChangePropIfDifferent(value, _taxusage)
        End Set
    End Property


    Private _taxenvironment As String = " - "
    Public Property TaxEnvironment() As String
        Get
            Return _taxenvironment
        End Get
        Set(ByVal value As String)
            ChangePropIfDifferent(value, _taxenvironment)
        End Set
    End Property






#Region "Tempdata"
    Private _casenr As String = String.Empty
    Public Property CaseNr() As String
        Get
            Return _casenr
        End Get
        Set(ByVal value As String)
            ChangePropIfDifferent(value, _casenr)
        End Set
    End Property


    Private _customernr As String = String.Empty
    Public Property CustomerNr() As String
        Get
            Return _customernr
        End Get
        Set(ByVal value As String)
            ChangePropIfDifferent(value, _customernr)
        End Set
    End Property


    Private _deliverynr As String = String.Empty
    Public Property DeliveryNr() As String
        Get
            Return _deliverynr
        End Get
        Set(ByVal value As String)
            ChangePropIfDifferent(value, _deliverynr)
        End Set
    End Property


    Private _ponr As String = String.Empty
    Public Property PONr() As String
        Get
            Return _ponr
        End Get
        Set(ByVal value As String)
            ChangePropIfDifferent(value, _ponr)
        End Set
    End Property


    Private _prodname As String = String.Empty
    Public Property PRODName() As String
        Get
            Return _prodname
        End Get
        Set(ByVal value As String)
            ChangePropIfDifferent(value, _prodname)
        End Set
    End Property


    Private _prodnr As String = String.Empty
    Public Property PRODNr() As String
        Get
            Return _prodnr
        End Get
        Set(ByVal value As String)
            ChangePropIfDifferent(value, _prodnr)
        End Set
    End Property

#End Region






#Region "Commands"

    Public ReadOnly Property CopyIVNToClipboardCommand As New RelayCommand(AddressOf CopyIVNToClipboardCommand_Execute, Function(o) __AvailableCountries(AvailableCountries.CurrentPosition) IsNot Nothing)

    Private Sub CopyIVNToClipboardCommand_Execute(obj As Object)

        Dim tmp_selectedText As String = __AvailableCountries(AvailableCountries.CurrentPosition).InvoiceText
        If PRODName IsNot Nothing Then tmp_selectedText = tmp_selectedText.Replace("""Device""", $"""{PRODName}""")
        If PRODNr IsNot Nothing Then tmp_selectedText = tmp_selectedText.Replace("""ProdNR""", $"""{PRODNr}""")
        If CaseNr IsNot Nothing Then tmp_selectedText = tmp_selectedText.Replace("""Case""", $"""{CaseNr}""")
        Clipboard.SetText(tmp_selectedText)

    End Sub



    Public ReadOnly Property CopyToClipboardCommand As New RelayCommand(AddressOf CopyToClipboardCommand_Execute)

    Private Sub CopyToClipboardCommand_Execute(obj As Object)
        Clipboard.SetText($"{obj.ToString}")
    End Sub



    Public ReadOnly Property ClearCommand As New RelayCommand(AddressOf ClearCommand_Execute, Function(o) CaseNr <> "" OrElse CustomerNr <> "" OrElse DeliveryNr <> "" OrElse PONr <> "" _
                                                                                              OrElse PRODName <> "" OrElse PRODNr <> Nothing)

    Private Sub ClearCommand_Execute(obj As Object)
        CaseNr = String.Empty : CustomerNr = String.Empty : DeliveryNr = String.Empty : PONr = String.Empty : PRODName = String.Empty : PRODNr = String.Empty
    End Sub
#End Region






#Region "Funktionen"

    Private Sub CalcResults()
        Dim tmp_DaybookExtension As String = String.Empty
        Dim tmp_DaybookLand As String = String.Empty
        Dim tmp_TaxEnvironment As String = String.Empty
        Dim tmp_TaxUsage As Boolean = False

        Dim SelectedCountry = __AvailableCountries(AvailableCountries.CurrentPosition)

        If SelectedCountry IsNot Nothing Then

            If TaxUse Then
                tmp_DaybookExtension = "-X"
                tmp_DaybookLand = SelectedCountry.CountryDayBookSet
                tmp_TaxUsage = True
                tmp_TaxEnvironment = SelectedCountry.CountryTaxEnvironment
            Else

                If SalesType Then
                    tmp_DaybookLand = "EMA-DE"
                    tmp_TaxEnvironment = SelectedCountry.CountryTaxEnvironment
                Else
                    tmp_DaybookLand = SelectedCountry.CountryDayBookSet
                    tmp_TaxEnvironment = "DCNO-ZZEU-S"
                End If

                If SelectedCountry.CountryName = "Germany" Then
                    tmp_TaxUsage = "1"
                    tmp_TaxEnvironment = "TCDE-DEU-S"
                End If
            End If

            DayBookSet = $"{tmp_DaybookLand}{tmp_DaybookExtension}"
            TaxUsage = tmp_TaxUsage
            TaxEnvironment = tmp_TaxEnvironment
        Else
            DayBookSet = " - " : TaxUsage = False : TaxEnvironment = " - "
        End If

        If IO.File.Exists(Application.Vat0) Then
            For Each l As String In IO.File.ReadLines(Application.Vat0) 'Vat 0 und 1 Sind die Pfade zu den Whitelist Textdateien, die, falls noch nichts existent im StartupPfad, leer generiert werden
                If l = CustomerNr Then
                    TaxUsage = "0"
                End If
            Next

            For Each l As String In IO.File.ReadLines(Application.Vat1)
                If l = CustomerNr Then
                    TaxUsage = "1"
                End If
            Next
        End If


    End Sub

#End Region


End Class